# ruff: noqa: F405
# this file is generated by build_network_protocol_files.py

from __future__ import annotations

from pathlib import Path

from chik._tests.util.build_network_protocol_files import get_network_protocol_filename
from chik._tests.util.network_protocol_data import *  # noqa: F403
from chik._tests.util.protocol_messages_json import *  # noqa: F403


def parse_blob(input_bytes: bytes) -> tuple[bytes, bytes]:
    size_bytes = input_bytes[:4]
    input_bytes = input_bytes[4:]
    size = int.from_bytes(size_bytes, "big")
    message_bytes = input_bytes[:size]
    input_bytes = input_bytes[size:]
    return (message_bytes, input_bytes)


def test_protocol_bytes() -> None:
    filename: Path = get_network_protocol_filename()
    assert filename.exists()
    with open(filename, "rb") as f:
        input_bytes = f.read()

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_0 = type(new_signage_point).from_bytes(message_bytes)
    assert message_0 == new_signage_point
    assert bytes(message_0) == bytes(new_signage_point)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_1 = type(declare_proof_of_space).from_bytes(message_bytes)
    assert message_1 == declare_proof_of_space
    assert bytes(message_1) == bytes(declare_proof_of_space)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_2 = type(request_signed_values).from_bytes(message_bytes)
    assert message_2 == request_signed_values
    assert bytes(message_2) == bytes(request_signed_values)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_3 = type(farming_info).from_bytes(message_bytes)
    assert message_3 == farming_info
    assert bytes(message_3) == bytes(farming_info)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_4 = type(signed_values).from_bytes(message_bytes)
    assert message_4 == signed_values
    assert bytes(message_4) == bytes(signed_values)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_5 = type(new_peak).from_bytes(message_bytes)
    assert message_5 == new_peak
    assert bytes(message_5) == bytes(new_peak)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_6 = type(new_transaction).from_bytes(message_bytes)
    assert message_6 == new_transaction
    assert bytes(message_6) == bytes(new_transaction)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_7 = type(request_transaction).from_bytes(message_bytes)
    assert message_7 == request_transaction
    assert bytes(message_7) == bytes(request_transaction)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_8 = type(respond_transaction).from_bytes(message_bytes)
    assert message_8 == respond_transaction
    assert bytes(message_8) == bytes(respond_transaction)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_9 = type(request_proof_of_weight).from_bytes(message_bytes)
    assert message_9 == request_proof_of_weight
    assert bytes(message_9) == bytes(request_proof_of_weight)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_10 = type(respond_proof_of_weight).from_bytes(message_bytes)
    assert message_10 == respond_proof_of_weight
    assert bytes(message_10) == bytes(respond_proof_of_weight)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_11 = type(request_block).from_bytes(message_bytes)
    assert message_11 == request_block
    assert bytes(message_11) == bytes(request_block)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_12 = type(reject_block).from_bytes(message_bytes)
    assert message_12 == reject_block
    assert bytes(message_12) == bytes(reject_block)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_13 = type(request_blocks).from_bytes(message_bytes)
    assert message_13 == request_blocks
    assert bytes(message_13) == bytes(request_blocks)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_14 = type(respond_blocks).from_bytes(message_bytes)
    assert message_14 == respond_blocks
    assert bytes(message_14) == bytes(respond_blocks)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_15 = type(reject_blocks).from_bytes(message_bytes)
    assert message_15 == reject_blocks
    assert bytes(message_15) == bytes(reject_blocks)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_16 = type(respond_block).from_bytes(message_bytes)
    assert message_16 == respond_block
    assert bytes(message_16) == bytes(respond_block)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_17 = type(new_unfinished_block).from_bytes(message_bytes)
    assert message_17 == new_unfinished_block
    assert bytes(message_17) == bytes(new_unfinished_block)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_18 = type(request_unfinished_block).from_bytes(message_bytes)
    assert message_18 == request_unfinished_block
    assert bytes(message_18) == bytes(request_unfinished_block)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_19 = type(respond_unfinished_block).from_bytes(message_bytes)
    assert message_19 == respond_unfinished_block
    assert bytes(message_19) == bytes(respond_unfinished_block)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_20 = type(new_signage_point_or_end_of_subslot).from_bytes(message_bytes)
    assert message_20 == new_signage_point_or_end_of_subslot
    assert bytes(message_20) == bytes(new_signage_point_or_end_of_subslot)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_21 = type(request_signage_point_or_end_of_subslot).from_bytes(message_bytes)
    assert message_21 == request_signage_point_or_end_of_subslot
    assert bytes(message_21) == bytes(request_signage_point_or_end_of_subslot)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_22 = type(respond_signage_point).from_bytes(message_bytes)
    assert message_22 == respond_signage_point
    assert bytes(message_22) == bytes(respond_signage_point)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_23 = type(respond_end_of_subslot).from_bytes(message_bytes)
    assert message_23 == respond_end_of_subslot
    assert bytes(message_23) == bytes(respond_end_of_subslot)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_24 = type(request_mempool_transaction).from_bytes(message_bytes)
    assert message_24 == request_mempool_transaction
    assert bytes(message_24) == bytes(request_mempool_transaction)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_25 = type(new_compact_vdf).from_bytes(message_bytes)
    assert message_25 == new_compact_vdf
    assert bytes(message_25) == bytes(new_compact_vdf)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_26 = type(request_compact_vdf).from_bytes(message_bytes)
    assert message_26 == request_compact_vdf
    assert bytes(message_26) == bytes(request_compact_vdf)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_27 = type(respond_compact_vdf).from_bytes(message_bytes)
    assert message_27 == respond_compact_vdf
    assert bytes(message_27) == bytes(respond_compact_vdf)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_28 = type(request_peers).from_bytes(message_bytes)
    assert message_28 == request_peers
    assert bytes(message_28) == bytes(request_peers)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_29 = type(respond_peers).from_bytes(message_bytes)
    assert message_29 == respond_peers
    assert bytes(message_29) == bytes(respond_peers)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_30 = type(new_unfinished_block2).from_bytes(message_bytes)
    assert message_30 == new_unfinished_block2
    assert bytes(message_30) == bytes(new_unfinished_block2)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_31 = type(request_unfinished_block2).from_bytes(message_bytes)
    assert message_31 == request_unfinished_block2
    assert bytes(message_31) == bytes(request_unfinished_block2)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_32 = type(request_puzzle_solution).from_bytes(message_bytes)
    assert message_32 == request_puzzle_solution
    assert bytes(message_32) == bytes(request_puzzle_solution)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_33 = type(puzzle_solution_response).from_bytes(message_bytes)
    assert message_33 == puzzle_solution_response
    assert bytes(message_33) == bytes(puzzle_solution_response)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_34 = type(respond_puzzle_solution).from_bytes(message_bytes)
    assert message_34 == respond_puzzle_solution
    assert bytes(message_34) == bytes(respond_puzzle_solution)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_35 = type(reject_puzzle_solution).from_bytes(message_bytes)
    assert message_35 == reject_puzzle_solution
    assert bytes(message_35) == bytes(reject_puzzle_solution)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_36 = type(send_transaction).from_bytes(message_bytes)
    assert message_36 == send_transaction
    assert bytes(message_36) == bytes(send_transaction)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_37 = type(transaction_ack).from_bytes(message_bytes)
    assert message_37 == transaction_ack
    assert bytes(message_37) == bytes(transaction_ack)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_38 = type(new_peak_wallet).from_bytes(message_bytes)
    assert message_38 == new_peak_wallet
    assert bytes(message_38) == bytes(new_peak_wallet)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_39 = type(request_block_header).from_bytes(message_bytes)
    assert message_39 == request_block_header
    assert bytes(message_39) == bytes(request_block_header)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_40 = type(request_block_headers).from_bytes(message_bytes)
    assert message_40 == request_block_headers
    assert bytes(message_40) == bytes(request_block_headers)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_41 = type(respond_header_block).from_bytes(message_bytes)
    assert message_41 == respond_header_block
    assert bytes(message_41) == bytes(respond_header_block)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_42 = type(respond_block_headers).from_bytes(message_bytes)
    assert message_42 == respond_block_headers
    assert bytes(message_42) == bytes(respond_block_headers)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_43 = type(reject_header_request).from_bytes(message_bytes)
    assert message_43 == reject_header_request
    assert bytes(message_43) == bytes(reject_header_request)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_44 = type(request_removals).from_bytes(message_bytes)
    assert message_44 == request_removals
    assert bytes(message_44) == bytes(request_removals)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_45 = type(respond_removals).from_bytes(message_bytes)
    assert message_45 == respond_removals
    assert bytes(message_45) == bytes(respond_removals)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_46 = type(reject_removals_request).from_bytes(message_bytes)
    assert message_46 == reject_removals_request
    assert bytes(message_46) == bytes(reject_removals_request)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_47 = type(request_additions).from_bytes(message_bytes)
    assert message_47 == request_additions
    assert bytes(message_47) == bytes(request_additions)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_48 = type(respond_additions).from_bytes(message_bytes)
    assert message_48 == respond_additions
    assert bytes(message_48) == bytes(respond_additions)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_49 = type(reject_additions).from_bytes(message_bytes)
    assert message_49 == reject_additions
    assert bytes(message_49) == bytes(reject_additions)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_50 = type(request_header_blocks).from_bytes(message_bytes)
    assert message_50 == request_header_blocks
    assert bytes(message_50) == bytes(request_header_blocks)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_51 = type(reject_header_blocks).from_bytes(message_bytes)
    assert message_51 == reject_header_blocks
    assert bytes(message_51) == bytes(reject_header_blocks)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_52 = type(respond_header_blocks).from_bytes(message_bytes)
    assert message_52 == respond_header_blocks
    assert bytes(message_52) == bytes(respond_header_blocks)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_53 = type(coin_state).from_bytes(message_bytes)
    assert message_53 == coin_state
    assert bytes(message_53) == bytes(coin_state)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_54 = type(register_for_ph_updates).from_bytes(message_bytes)
    assert message_54 == register_for_ph_updates
    assert bytes(message_54) == bytes(register_for_ph_updates)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_55 = type(reject_block_headers).from_bytes(message_bytes)
    assert message_55 == reject_block_headers
    assert bytes(message_55) == bytes(reject_block_headers)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_56 = type(respond_to_ph_updates).from_bytes(message_bytes)
    assert message_56 == respond_to_ph_updates
    assert bytes(message_56) == bytes(respond_to_ph_updates)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_57 = type(register_for_coin_updates).from_bytes(message_bytes)
    assert message_57 == register_for_coin_updates
    assert bytes(message_57) == bytes(register_for_coin_updates)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_58 = type(respond_to_coin_updates).from_bytes(message_bytes)
    assert message_58 == respond_to_coin_updates
    assert bytes(message_58) == bytes(respond_to_coin_updates)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_59 = type(coin_state_update).from_bytes(message_bytes)
    assert message_59 == coin_state_update
    assert bytes(message_59) == bytes(coin_state_update)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_60 = type(request_children).from_bytes(message_bytes)
    assert message_60 == request_children
    assert bytes(message_60) == bytes(request_children)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_61 = type(respond_children).from_bytes(message_bytes)
    assert message_61 == respond_children
    assert bytes(message_61) == bytes(respond_children)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_62 = type(request_ses_info).from_bytes(message_bytes)
    assert message_62 == request_ses_info
    assert bytes(message_62) == bytes(request_ses_info)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_63 = type(respond_ses_info).from_bytes(message_bytes)
    assert message_63 == respond_ses_info
    assert bytes(message_63) == bytes(respond_ses_info)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_64 = type(coin_state_filters).from_bytes(message_bytes)
    assert message_64 == coin_state_filters
    assert bytes(message_64) == bytes(coin_state_filters)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_65 = type(request_remove_puzzle_subscriptions).from_bytes(message_bytes)
    assert message_65 == request_remove_puzzle_subscriptions
    assert bytes(message_65) == bytes(request_remove_puzzle_subscriptions)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_66 = type(respond_remove_puzzle_subscriptions).from_bytes(message_bytes)
    assert message_66 == respond_remove_puzzle_subscriptions
    assert bytes(message_66) == bytes(respond_remove_puzzle_subscriptions)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_67 = type(request_remove_coin_subscriptions).from_bytes(message_bytes)
    assert message_67 == request_remove_coin_subscriptions
    assert bytes(message_67) == bytes(request_remove_coin_subscriptions)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_68 = type(respond_remove_coin_subscriptions).from_bytes(message_bytes)
    assert message_68 == respond_remove_coin_subscriptions
    assert bytes(message_68) == bytes(respond_remove_coin_subscriptions)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_69 = type(request_puzzle_state).from_bytes(message_bytes)
    assert message_69 == request_puzzle_state
    assert bytes(message_69) == bytes(request_puzzle_state)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_70 = type(reject_puzzle_state).from_bytes(message_bytes)
    assert message_70 == reject_puzzle_state
    assert bytes(message_70) == bytes(reject_puzzle_state)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_71 = type(respond_puzzle_state).from_bytes(message_bytes)
    assert message_71 == respond_puzzle_state
    assert bytes(message_71) == bytes(respond_puzzle_state)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_72 = type(request_coin_state).from_bytes(message_bytes)
    assert message_72 == request_coin_state
    assert bytes(message_72) == bytes(request_coin_state)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_73 = type(respond_coin_state).from_bytes(message_bytes)
    assert message_73 == respond_coin_state
    assert bytes(message_73) == bytes(respond_coin_state)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_74 = type(reject_coin_state).from_bytes(message_bytes)
    assert message_74 == reject_coin_state
    assert bytes(message_74) == bytes(reject_coin_state)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_75 = type(request_cost_info).from_bytes(message_bytes)
    assert message_75 == request_cost_info
    assert bytes(message_75) == bytes(request_cost_info)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_76 = type(respond_cost_info).from_bytes(message_bytes)
    assert message_76 == respond_cost_info
    assert bytes(message_76) == bytes(respond_cost_info)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_77 = type(pool_difficulty).from_bytes(message_bytes)
    assert message_77 == pool_difficulty
    assert bytes(message_77) == bytes(pool_difficulty)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_78 = type(harvester_handhsake).from_bytes(message_bytes)
    assert message_78 == harvester_handhsake
    assert bytes(message_78) == bytes(harvester_handhsake)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_79 = type(new_signage_point_harvester).from_bytes(message_bytes)
    assert message_79 == new_signage_point_harvester
    assert bytes(message_79) == bytes(new_signage_point_harvester)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_80 = type(new_proof_of_space).from_bytes(message_bytes)
    assert message_80 == new_proof_of_space
    assert bytes(message_80) == bytes(new_proof_of_space)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_81 = type(request_signatures).from_bytes(message_bytes)
    assert message_81 == request_signatures
    assert bytes(message_81) == bytes(request_signatures)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_82 = type(respond_signatures).from_bytes(message_bytes)
    assert message_82 == respond_signatures
    assert bytes(message_82) == bytes(respond_signatures)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_83 = type(plot).from_bytes(message_bytes)
    assert message_83 == plot
    assert bytes(message_83) == bytes(plot)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_84 = type(request_plots).from_bytes(message_bytes)
    assert message_84 == request_plots
    assert bytes(message_84) == bytes(request_plots)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_85 = type(respond_plots).from_bytes(message_bytes)
    assert message_85 == respond_plots
    assert bytes(message_85) == bytes(respond_plots)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_86 = type(request_peers_introducer).from_bytes(message_bytes)
    assert message_86 == request_peers_introducer
    assert bytes(message_86) == bytes(request_peers_introducer)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_87 = type(respond_peers_introducer).from_bytes(message_bytes)
    assert message_87 == respond_peers_introducer
    assert bytes(message_87) == bytes(respond_peers_introducer)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_88 = type(authentication_payload).from_bytes(message_bytes)
    assert message_88 == authentication_payload
    assert bytes(message_88) == bytes(authentication_payload)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_89 = type(get_pool_info_response).from_bytes(message_bytes)
    assert message_89 == get_pool_info_response
    assert bytes(message_89) == bytes(get_pool_info_response)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_90 = type(post_partial_payload).from_bytes(message_bytes)
    assert message_90 == post_partial_payload
    assert bytes(message_90) == bytes(post_partial_payload)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_91 = type(post_partial_request).from_bytes(message_bytes)
    assert message_91 == post_partial_request
    assert bytes(message_91) == bytes(post_partial_request)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_92 = type(post_partial_response).from_bytes(message_bytes)
    assert message_92 == post_partial_response
    assert bytes(message_92) == bytes(post_partial_response)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_93 = type(get_farmer_response).from_bytes(message_bytes)
    assert message_93 == get_farmer_response
    assert bytes(message_93) == bytes(get_farmer_response)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_94 = type(post_farmer_payload).from_bytes(message_bytes)
    assert message_94 == post_farmer_payload
    assert bytes(message_94) == bytes(post_farmer_payload)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_95 = type(post_farmer_request).from_bytes(message_bytes)
    assert message_95 == post_farmer_request
    assert bytes(message_95) == bytes(post_farmer_request)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_96 = type(post_farmer_response).from_bytes(message_bytes)
    assert message_96 == post_farmer_response
    assert bytes(message_96) == bytes(post_farmer_response)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_97 = type(put_farmer_payload).from_bytes(message_bytes)
    assert message_97 == put_farmer_payload
    assert bytes(message_97) == bytes(put_farmer_payload)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_98 = type(put_farmer_request).from_bytes(message_bytes)
    assert message_98 == put_farmer_request
    assert bytes(message_98) == bytes(put_farmer_request)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_99 = type(put_farmer_response).from_bytes(message_bytes)
    assert message_99 == put_farmer_response
    assert bytes(message_99) == bytes(put_farmer_response)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_100 = type(error_response).from_bytes(message_bytes)
    assert message_100 == error_response
    assert bytes(message_100) == bytes(error_response)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_101 = type(new_peak_timelord).from_bytes(message_bytes)
    assert message_101 == new_peak_timelord
    assert bytes(message_101) == bytes(new_peak_timelord)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_102 = type(new_unfinished_block_timelord).from_bytes(message_bytes)
    assert message_102 == new_unfinished_block_timelord
    assert bytes(message_102) == bytes(new_unfinished_block_timelord)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_103 = type(new_infusion_point_vdf).from_bytes(message_bytes)
    assert message_103 == new_infusion_point_vdf
    assert bytes(message_103) == bytes(new_infusion_point_vdf)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_104 = type(new_signage_point_vdf).from_bytes(message_bytes)
    assert message_104 == new_signage_point_vdf
    assert bytes(message_104) == bytes(new_signage_point_vdf)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_105 = type(new_end_of_sub_slot_bundle).from_bytes(message_bytes)
    assert message_105 == new_end_of_sub_slot_bundle
    assert bytes(message_105) == bytes(new_end_of_sub_slot_bundle)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_106 = type(request_compact_proof_of_time).from_bytes(message_bytes)
    assert message_106 == request_compact_proof_of_time
    assert bytes(message_106) == bytes(request_compact_proof_of_time)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_107 = type(respond_compact_proof_of_time).from_bytes(message_bytes)
    assert message_107 == respond_compact_proof_of_time
    assert bytes(message_107) == bytes(respond_compact_proof_of_time)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_108 = type(error_without_data).from_bytes(message_bytes)
    assert message_108 == error_without_data
    assert bytes(message_108) == bytes(error_without_data)

    message_bytes, input_bytes = parse_blob(input_bytes)
    message_109 = type(error_with_data).from_bytes(message_bytes)
    assert message_109 == error_with_data
    assert bytes(message_109) == bytes(error_with_data)

    assert input_bytes == b""



from __future__ import annotations

import asyncio
import logging
import os
import pathlib
import signal
import sys
import time
from collections.abc import AsyncIterator
from contextlib import asynccontextmanager
from dataclasses import dataclass, field
from types import FrameType
from typing import Any, Optional

from chik.server.signal_handlers import SignalHandlers
from chik.util.chik_logging import initialize_logging
from chik.util.config import load_config
from chik.util.default_root import resolve_root_path
from chik.util.network import resolve
from chik.util.setproctitle import setproctitle

log = logging.getLogger(__name__)


@dataclass
class VDFClientProcessMgr:
    lock: asyncio.Lock = field(default_factory=asyncio.Lock)
    stopped: bool = False
    active_processes: list[asyncio.subprocess.Process] = field(default_factory=list)

    async def remove_process(self, proc: asyncio.subprocess.Process) -> None:
        async with self.lock:
            try:
                self.active_processes.remove(proc)
            except ValueError:
                pass

    async def add_process(self, proc: asyncio.subprocess.Process) -> None:
        async with self.lock:
            self.active_processes.append(proc)

    async def kill_processes(self) -> None:
        async with self.lock:
            self.stopped = True
            for process in self.active_processes:
                try:
                    process.kill()
                    await process.wait()
                    if sys.version_info < (3, 11, 1):
                        # hack to avoid `Event loop is closed` errors (fixed in python 3.11.1)
                        # https://github.com/python/cpython/issues/88050
                        process._transport.close()  # type: ignore [attr-defined]
                except (ProcessLookupError, AttributeError):
                    pass
            self.active_processes.clear()

    @asynccontextmanager
    async def manage_proc(self, proc: asyncio.subprocess.Process) -> AsyncIterator[None]:
        await self.add_process(proc)
        try:
            yield
        finally:
            await self.remove_process(proc)


def find_vdf_client() -> pathlib.Path:
    try:
        import chikvdf
    except ImportError:
        raise Exception("Cannot import chikvdf package")

    file_string = getattr(chikvdf, "__file__", None)
    if file_string is None:
        raise Exception("Cannot find chikvdf package location")

    location = pathlib.Path(file_string).parent
    p = location.joinpath("vdf_client")
    if p.is_file():
        return p
    raise FileNotFoundError("Cannot find vdf_client binary. Is Timelord installed? See install-timelord.sh")


async def spawn_process(
    host: str,
    port: int,
    counter: int,
    process_mgr: VDFClientProcessMgr,
    *,
    prefer_ipv6: bool,
) -> None:
    path_to_vdf_client = find_vdf_client()
    first_10_seconds = True
    start_time = time.time()
    while not process_mgr.stopped:
        try:
            dirname = path_to_vdf_client.parent
            basename = path_to_vdf_client.name
            resolved = await resolve(host, prefer_ipv6=prefer_ipv6)
            proc = await asyncio.create_subprocess_exec(
                os.fspath(basename),
                str(resolved),
                str(port),
                str(counter),
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
                env={"PATH": os.fspath(dirname)},
            )
        except Exception as e:
            log.warning(f"Exception while spawning process {counter}: {(e)}")
            continue

        async with process_mgr.manage_proc(proc):
            while True:
                if proc.stdout is None or proc.stderr is None:
                    break
                if proc.stdout.at_eof() and proc.stderr.at_eof():
                    break
                stdout = (await proc.stdout.readline()).decode().rstrip()
                if stdout:
                    log.info(f"VDF client {counter}: {stdout}")
                stderr = (await proc.stderr.readline()).decode().rstrip()
                if stderr:
                    if first_10_seconds:
                        if time.time() - start_time > 10:
                            first_10_seconds = False
                    else:
                        log.error(f"VDF client {counter}: {stderr}")

                await asyncio.sleep(0.1)

            await proc.communicate()


async def spawn_all_processes(config: dict, net_config: dict, process_mgr: VDFClientProcessMgr):
    await asyncio.sleep(5)
    hostname = net_config["self_hostname"] if "host" not in config else config["host"]
    port = config["port"]
    process_count = config["process_count"]
    if process_count == 0:
        log.info("Process_count set to 0, stopping TLauncher.")
        return
    awaitables = [
        spawn_process(
            hostname,
            port,
            i,
            process_mgr,
            prefer_ipv6=net_config.get("prefer_ipv6", False),
        )
        for i in range(process_count)
    ]
    await asyncio.gather(*awaitables)


async def async_main(config: dict[str, Any], net_config: dict[str, Any]) -> None:
    process_mgr = VDFClientProcessMgr()

    async def stop(
        signal_: signal.Signals,
        stack_frame: Optional[FrameType],
        loop: asyncio.AbstractEventLoop,
    ) -> None:
        await process_mgr.kill_processes()

    async with SignalHandlers.manage() as signal_handlers:
        signal_handlers.setup_async_signal_handler(handler=stop)

        try:
            await spawn_all_processes(config, net_config, process_mgr)
        finally:
            log.info("Launcher fully closed.")


def main():
    if os.name == "nt":
        log.info("Timelord launcher not supported on Windows.")
        return 1
    root_path = resolve_root_path(override=None)

    setproctitle("chik_timelord_launcher")
    net_config = load_config(root_path, "config.yaml")
    config = net_config["timelord_launcher"]
    initialize_logging("TLauncher", config["logging"], root_path)

    asyncio.run(async_main(config=config, net_config=net_config))


if __name__ == "__main__":
    sys.exit(main())



# Package: utils

from __future__ import annotations

from dataclasses import replace
from typing import Any


def recursive_replace(root_obj: Any, replace_str: str, replace_with: Any) -> Any:
    split_str = replace_str.split(".")
    if len(split_str) == 1:
        # This check is here to support native types (implemented in Rust
        # in chik_rs) that aren't dataclasses. They instead implement a
        # replace() method in their python bindings.
        if hasattr(root_obj, "replace"):
            return root_obj.replace(**{split_str[0]: replace_with})
        else:
            return replace(root_obj, **{split_str[0]: replace_with})
    sub_obj = recursive_replace(getattr(root_obj, split_str[0]), ".".join(split_str[1:]), replace_with)
    # See comment above
    if hasattr(root_obj, "replace"):
        return root_obj.replace(**{split_str[0]: sub_obj})
    else:
        return replace(root_obj, **{split_str[0]: sub_obj})



from __future__ import annotations

import asyncio
import logging
import signal
import sys
from dataclasses import dataclass, field
from pathlib import Path
from types import FrameType
from typing import Any, Optional

import click
from aiohttp import web

from chik.data_layer.download_data import is_filename_valid
from chik.server.signal_handlers import SignalHandlers
from chik.server.upnp import UPnP
from chik.util.chik_logging import initialize_logging
from chik.util.config import load_config
from chik.util.default_root import DEFAULT_ROOT_PATH
from chik.util.network import WebServer
from chik.util.path import path_from_root
from chik.util.setproctitle import setproctitle

# from chik.cmds.chik import monkey_patch_click


# See: https://bugs.python.org/issue29288
"".encode("idna")

SERVICE_NAME = "data_layer"
log = logging.getLogger(__name__)


@dataclass
class DataLayerServer:
    root_path: Path
    config: dict[str, Any]
    log: logging.Logger
    shutdown_event: asyncio.Event
    webserver: Optional[WebServer] = None
    upnp: UPnP = field(default_factory=UPnP)

    async def start(self, signal_handlers: SignalHandlers) -> None:
        if self.webserver is not None:
            raise RuntimeError("DataLayerServer already started")

        signal_handlers.setup_sync_signal_handler(handler=self._accept_signal)

        self.log.info("Starting Data Layer HTTP Server.")

        self.host_ip = self.config["host_ip"]
        self.port = self.config["host_port"]

        # Setup UPnP for the data_layer_service port
        self.upnp.setup()
        self.upnp.remap(self.port)

        server_files_replaced: str = self.config.get(
            "server_files_location", "data_layer/db/server_files_location_CHALLENGE"
        ).replace("CHALLENGE", self.config["selected_network"])
        self.server_dir = path_from_root(self.root_path, server_files_replaced)

        self.webserver = await WebServer.create(
            hostname=self.host_ip,
            port=self.port,
            routes=[
                web.get("/{filename}", self.file_handler),
                web.get("/{tree_id}/{filename}", self.folder_handler),
            ],
        )
        self.log.info("Started Data Layer HTTP Server.")

    def close(self) -> None:
        self.shutdown_event.set()
        self.upnp.release(self.port)
        # UPnP.shutdown() is a blocking call, waiting for the UPnP thread to exit
        self.upnp.shutdown()

        if self.webserver is not None:
            self.webserver.close()

        self.log.info("Stop triggered for Data Layer HTTP Server.")

    async def await_closed(self) -> None:
        self.log.info("Wait for Data Layer HTTP Server shutdown.")
        if self.webserver is not None:
            await self.webserver.await_closed()
            self.webserver = None

    async def file_handler(self, request: web.Request) -> web.Response:
        filename = request.match_info["filename"]
        if not is_filename_valid(filename):
            raise Exception("Invalid file format requested.")
        file_path = self.server_dir.joinpath(filename)
        with open(file_path, "rb") as reader:
            content = reader.read()
        response = web.Response(
            content_type="application/octet-stream",
            headers={"Content-Disposition": f"attachment;filename={filename}"},
            body=content,
        )
        return response

    async def folder_handler(self, request: web.Request) -> web.Response:
        tree_id = request.match_info["tree_id"]
        filename = request.match_info["filename"]
        if not is_filename_valid(tree_id + "-" + filename):
            raise Exception("Invalid file format requested.")
        file_path = self.server_dir.joinpath(tree_id).joinpath(filename)
        with open(file_path, "rb") as reader:
            content = reader.read()
        response = web.Response(
            content_type="application/octet-stream",
            headers={"Content-Disposition": f"attachment;filename={filename}"},
            body=content,
        )
        return response

    def _accept_signal(
        self,
        signal_: signal.Signals,
        stack_frame: Optional[FrameType],
        loop: asyncio.AbstractEventLoop,
    ) -> None:
        self.log.info("Received signal %s (%s), shutting down.", signal_.name, signal_.value)

        self.close()


async def async_start(root_path: Path) -> int:
    shutdown_event = asyncio.Event()

    dl_config = load_config(
        root_path=root_path,
        filename="config.yaml",
        sub_config=SERVICE_NAME,
        fill_missing_services=True,
    )
    setproctitle("data_layer_http")
    initialize_logging(
        service_name="data_layer_http",
        logging_config=dl_config["logging"],
        root_path=root_path,
    )

    data_layer_server = DataLayerServer(root_path, dl_config, log, shutdown_event)
    async with SignalHandlers.manage() as signal_handlers:
        await data_layer_server.start(signal_handlers=signal_handlers)
        await shutdown_event.wait()
        await data_layer_server.await_closed()

    return 0


@click.command()
@click.option(
    "-r",
    "--root-path",
    type=click.Path(exists=True, writable=True, file_okay=False),
    default=DEFAULT_ROOT_PATH,
    show_default=True,
    help="Config file root",
)
def main(root_path: str = str(DEFAULT_ROOT_PATH)) -> int:
    return asyncio.run(async_start(Path(root_path)))


if __name__ == "__main__":
    sys.exit(main())



from __future__ import annotations

import asyncio
import json
import ssl
from collections.abc import AsyncIterator
from contextlib import asynccontextmanager
from pathlib import Path
from typing import Any, Optional

import aiohttp

from chik.util.ints import uint32
from chik.util.json_util import dict_to_json_str
from chik.util.task_referencer import create_referenced_task
from chik.util.ws_message import WsRpcMessage, create_payload_dict


class DaemonProxy:
    def __init__(
        self,
        uri: str,
        ssl_context: Optional[ssl.SSLContext],
        heartbeat: int,
        max_message_size: int = 50 * 1000 * 1000,
    ):
        self._uri = uri
        self._request_dict: dict[str, asyncio.Event] = {}
        self.response_dict: dict[str, WsRpcMessage] = {}
        self.ssl_context = ssl_context
        self.heartbeat = heartbeat
        self.client_session: Optional[aiohttp.ClientSession] = None
        self.websocket: Optional[aiohttp.ClientWebSocketResponse] = None
        self.max_message_size = max_message_size

    def format_request(self, command: str, data: dict[str, Any]) -> WsRpcMessage:
        request = create_payload_dict(command, data, "client", "daemon")
        return request

    async def start(self, wait_for_start: bool = False) -> None:
        self.client_session = aiohttp.ClientSession()

        connect_backoff = 2
        while (self.websocket is None or self.websocket.closed) and connect_backoff <= 60:
            try:
                self.websocket = await self.client_session.ws_connect(
                    self._uri,
                    autoclose=True,
                    autoping=True,
                    heartbeat=self.heartbeat,
                    ssl=self.ssl_context if self.ssl_context is not None else True,
                    max_msg_size=self.max_message_size,
                )
                break
            except aiohttp.ClientError:
                if not wait_for_start:
                    break
                await asyncio.sleep(connect_backoff)
                connect_backoff *= 2

        if self.websocket is None or self.websocket.closed:
            await self.close()
            raise Exception("Failed to connect to daemon")

        async def listener_task() -> None:
            try:
                await self.listener()
            finally:
                await self.close()

        create_referenced_task(listener_task(), known_unreferenced=True)
        await asyncio.sleep(1)

    async def listener(self) -> None:
        if self.websocket is None:
            raise TypeError("Websocket is None in listener!")
        while True:
            message = await self.websocket.receive()
            if message.type == aiohttp.WSMsgType.TEXT:
                decoded: WsRpcMessage = json.loads(message.data)
                request_id = decoded["request_id"]

                if request_id in self._request_dict:
                    self.response_dict[request_id] = decoded
                    self._request_dict[request_id].set()
            else:
                return None

    async def _get(self, request: WsRpcMessage) -> WsRpcMessage:
        request_id = request["request_id"]
        self._request_dict[request_id] = asyncio.Event()
        string = dict_to_json_str(request)
        if self.websocket is None or self.websocket.closed:
            raise Exception("Websocket is not connected")
        create_referenced_task(self.websocket.send_str(string), known_unreferenced=True)
        try:
            await asyncio.wait_for(self._request_dict[request_id].wait(), timeout=30)
            self._request_dict.pop(request_id)
            response: WsRpcMessage = self.response_dict[request_id]
            self.response_dict.pop(request_id)
            return response
        except asyncio.TimeoutError:
            self._request_dict.pop(request_id)
            raise Exception(f"No response from daemon for request_id: {request_id}")

    async def get_version(self) -> WsRpcMessage:
        data: dict[str, Any] = {}
        request = self.format_request("get_version", data)
        response = await self._get(request)
        return response

    async def get_network_info(self) -> WsRpcMessage:
        data: dict[str, Any] = {}
        request = self.format_request("get_network_info", data)
        response = await self._get(request)
        return response

    async def start_service(self, service_name: str) -> WsRpcMessage:
        data = {"service": service_name}
        request = self.format_request("start_service", data)
        response = await self._get(request)
        return response

    async def stop_service(self, service_name: str, delay_before_kill: int = 15) -> WsRpcMessage:
        data = {"service": service_name}
        request = self.format_request("stop_service", data)
        response = await self._get(request)
        return response

    async def is_running(self, service_name: str) -> bool:
        data = {"service": service_name}
        request = self.format_request("is_running", data)
        response = await self._get(request)
        if "is_running" in response["data"]:
            return bool(response["data"]["is_running"])
        return False

    async def is_keyring_locked(self) -> bool:
        data: dict[str, Any] = {}
        request = self.format_request("is_keyring_locked", data)
        response = await self._get(request)
        if "is_keyring_locked" in response["data"]:
            return bool(response["data"]["is_keyring_locked"])
        return False

    async def unlock_keyring(self, passphrase: str) -> WsRpcMessage:
        data = {"key": passphrase}
        request = self.format_request("unlock_keyring", data)
        response = await self._get(request)
        return response

    async def ping(self) -> WsRpcMessage:
        request = self.format_request("ping", {})
        response = await self._get(request)
        return response

    async def close(self) -> None:
        if self.websocket is not None:
            await self.websocket.close()
        if self.client_session is not None:
            await self.client_session.close()

    async def exit(self) -> WsRpcMessage:
        request = self.format_request("exit", {})
        return await self._get(request)

    async def get_keys_for_plotting(self, fingerprints: Optional[list[uint32]] = None) -> WsRpcMessage:
        data = {"fingerprints": fingerprints} if fingerprints else {}
        request = self.format_request("get_keys_for_plotting", data)
        response = await self._get(request)
        return response


async def connect_to_daemon(
    self_hostname: str,
    daemon_port: int,
    max_message_size: int,
    ssl_context: ssl.SSLContext,
    heartbeat: int,
    wait_for_start: bool = False,
) -> DaemonProxy:
    """
    Connect to the local daemon.
    """

    client = DaemonProxy(
        f"wss://{self_hostname}:{daemon_port}",
        ssl_context=ssl_context,
        max_message_size=max_message_size,
        heartbeat=heartbeat,
    )

    await client.start(wait_for_start=wait_for_start)
    return client


async def connect_to_daemon_and_validate(
    root_path: Path, config: dict[str, Any], quiet: bool = False, wait_for_start: bool = False
) -> Optional[DaemonProxy]:
    """
    Connect to the local daemon and do a ping to ensure that something is really
    there and running.
    """
    from chik.server.server import ssl_context_for_client

    try:
        daemon_max_message_size = config.get("daemon_max_message_size", 50 * 1000 * 1000)
        daemon_heartbeat = config.get("daemon_heartbeat", 300)
        crt_path = root_path / config["daemon_ssl"]["private_crt"]
        key_path = root_path / config["daemon_ssl"]["private_key"]
        ca_crt_path = root_path / config["private_ssl_ca"]["crt"]
        ca_key_path = root_path / config["private_ssl_ca"]["key"]
        ssl_context = ssl_context_for_client(ca_crt_path, ca_key_path, crt_path, key_path)
        connection = await connect_to_daemon(
            config["self_hostname"],
            config["daemon_port"],
            max_message_size=daemon_max_message_size,
            ssl_context=ssl_context,
            heartbeat=daemon_heartbeat,
            wait_for_start=wait_for_start,
        )
        r = await connection.ping()

        if "value" in r["data"] and r["data"]["value"] == "pong":
            return connection
    except Exception:
        if not quiet:
            print("Daemon not started yet")
        return None
    return None


@asynccontextmanager
async def acquire_connection_to_daemon(
    root_path: Path, config: dict[str, Any], quiet: bool = False
) -> AsyncIterator[Optional[DaemonProxy]]:
    """
    Asynchronous context manager which attempts to create a connection to the daemon.
    The connection object (DaemonProxy) is yielded to the caller. After the caller's
    block exits scope, execution resumes in this function, wherein the connection is
    closed.
    """

    daemon: Optional[DaemonProxy] = None
    try:
        daemon = await connect_to_daemon_and_validate(root_path, config, quiet=quiet)
        yield daemon  # <----
    except Exception as e:
        print(f"Exception occurred while communicating with the daemon: {e}")
    finally:
        if daemon is not None:
            await daemon.close()



from __future__ import annotations

import time
from typing import Any, Optional

from chik_rs import G1Element, G2Element, compute_merkle_set_root
from chikbip158 import PyBIP158

from chik.consensus.block_record import BlockRecord
from chik.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
from chik.consensus.coinbase import create_farmer_coin, create_pool_coin
from chik.consensus.constants import ConsensusConstants
from chik.consensus.full_block_to_block_record import block_to_block_record
from chik.full_node.bundle_tools import simple_solution_generator
from chik.simulator.block_tools import BlockTools, compute_additions_unchecked
from chik.types.blockchain_format.classgroup import ClassgroupElement
from chik.types.blockchain_format.coin import Coin, hash_coin_ids
from chik.types.blockchain_format.foliage import Foliage, FoliageBlockData, FoliageTransactionBlock, TransactionsInfo
from chik.types.blockchain_format.pool_target import PoolTarget
from chik.types.blockchain_format.proof_of_space import ProofOfSpace
from chik.types.blockchain_format.reward_chain_block import RewardChainBlock, RewardChainBlockUnfinished
from chik.types.blockchain_format.sized_bytes import bytes32, bytes100
from chik.types.blockchain_format.vdf import VDFInfo, VDFProof
from chik.types.full_block import FullBlock
from chik.types.generator_types import BlockGenerator
from chik.types.spend_bundle import SpendBundle
from chik.types.unfinished_block import UnfinishedBlock
from chik.util.block_cache import BlockCache
from chik.util.hash import std_hash
from chik.util.ints import uint8, uint32, uint64, uint128

DEFAULT_PROOF_OF_SPACE = ProofOfSpace(
    bytes32.zeros,
    G1Element(),
    None,
    G1Element(),
    uint8(20),
    bytes(32 * 5),
)
DEFAULT_VDF_INFO = VDFInfo(bytes32.zeros, uint64(1), ClassgroupElement(bytes100.zeros))
DEFAULT_VDF_PROOF = VDFProof(uint8(0), bytes(100), False)


class WalletBlockTools(BlockTools):
    """
    Tools to generate blocks for wallet testing.
    (Differs from standard block tools by patching away as much consensus logic as possible)
    """

    def get_consecutive_blocks(
        self,
        num_blocks: int,
        block_list_input: Optional[list[FullBlock]] = None,
        *,
        farmer_reward_puzzle_hash: Optional[bytes32] = None,
        pool_reward_puzzle_hash: Optional[bytes32] = None,
        transaction_data: Optional[SpendBundle] = None,
        genesis_timestamp: Optional[uint64] = None,
        **kwargs: Any,  # We're overriding so there's many arguments no longer used.
    ) -> list[FullBlock]:
        assert num_blocks > 0
        constants = self.constants

        if farmer_reward_puzzle_hash is None:
            farmer_reward_puzzle_hash = self.farmer_ph

        if block_list_input is None:
            block_list_input = []

        blocks: dict[bytes32, BlockRecord]
        if len(block_list_input) == 0:
            height_to_hash = {}
            blocks = {}
        elif block_list_input[-1].header_hash == self._block_cache_header:
            height_to_hash = self._block_cache_height_to_hash
            blocks = self._block_cache
        else:
            height_to_hash, _, blocks = load_block_list(block_list_input, constants)

        if len(block_list_input) > 0:
            latest_block: Optional[BlockRecord] = blocks[block_list_input[-1].header_hash]
            assert latest_block is not None
            assert latest_block.timestamp is not None
            last_timestamp = latest_block.timestamp
        else:
            latest_block = None
            last_timestamp = uint64((int(time.time()) if genesis_timestamp is None else genesis_timestamp) - 20)

        for _ in range(0, num_blocks):
            additions = []
            removals = []
            block_generator: Optional[BlockGenerator] = None
            if transaction_data is not None and len(block_list_input) > 0:
                additions = compute_additions_unchecked(transaction_data)
                removals = transaction_data.removals()
                block_generator = simple_solution_generator(transaction_data)
            pool_target = PoolTarget(
                pool_reward_puzzle_hash if pool_reward_puzzle_hash is not None else self.pool_ph, uint32(0)
            )

            (
                full_block,
                block_record,
                new_timestamp,
            ) = get_full_block_and_block_record(
                constants,
                blocks,
                last_timestamp,
                farmer_reward_puzzle_hash,
                pool_target,
                latest_block,
                block_generator,
                additions,
                removals,
            )

            transaction_data = None

            last_timestamp = uint64(new_timestamp)
            block_list_input.append(full_block)
            blocks[full_block.header_hash] = block_record
            height_to_hash[uint32(full_block.height)] = full_block.header_hash
            latest_block = block_record

        self._block_cache_header = block_list_input[-1].header_hash
        self._block_cache_height_to_hash = height_to_hash
        self._block_cache_difficulty = uint64(1)
        self._block_cache = blocks
        return block_list_input


def load_block_list(
    block_list: list[FullBlock], constants: ConsensusConstants
) -> tuple[dict[uint32, bytes32], uint64, dict[bytes32, BlockRecord]]:
    height_to_hash: dict[uint32, bytes32] = {}
    blocks: dict[bytes32, BlockRecord] = {}
    sub_slot_iters = constants.SUB_SLOT_ITERS_STARTING
    for full_block in block_list:
        if full_block.height != 0 and len(full_block.finished_sub_slots) > 0:
            if full_block.finished_sub_slots[0].challenge_chain.new_sub_slot_iters is not None:  # pragma: no cover
                sub_slot_iters = full_block.finished_sub_slots[0].challenge_chain.new_sub_slot_iters
        blocks[full_block.header_hash] = block_to_block_record(
            constants,
            BlockCache(blocks),
            uint64(1),
            full_block,
            sub_slot_iters,
        )
        height_to_hash[uint32(full_block.height)] = full_block.header_hash
    return height_to_hash, uint64(1), blocks


def finish_block(
    constants: ConsensusConstants,
    unfinished_block: UnfinishedBlock,
    prev_block: Optional[BlockRecord],
    blocks: dict[bytes32, BlockRecord],
) -> tuple[FullBlock, BlockRecord]:
    if prev_block is None:
        new_weight = uint128(1)
        new_height = uint32(0)
    else:
        new_weight = uint128(prev_block.weight + 1)
        new_height = uint32(prev_block.height + 1)

    full_block = FullBlock(
        [],
        RewardChainBlock(
            new_weight,
            new_height,
            uint128(1),
            uint8(1),
            bytes32.zeros,
            unfinished_block.reward_chain_block.proof_of_space,
            DEFAULT_VDF_INFO,
            G2Element(),
            DEFAULT_VDF_INFO,
            DEFAULT_VDF_INFO,
            G2Element(),
            DEFAULT_VDF_INFO,
            DEFAULT_VDF_INFO,
            prev_block is not None,
        ),
        DEFAULT_VDF_PROOF,
        DEFAULT_VDF_PROOF,
        DEFAULT_VDF_PROOF,
        DEFAULT_VDF_PROOF,
        DEFAULT_VDF_PROOF,
        unfinished_block.foliage,
        unfinished_block.foliage_transaction_block,
        unfinished_block.transactions_info,
        unfinished_block.transactions_generator,
        [],
    )

    block_record = block_to_block_record(constants, BlockCache(blocks), uint64(1), full_block, uint64(1))
    return full_block, block_record


def get_full_block_and_block_record(
    constants: ConsensusConstants,
    blocks: dict[bytes32, BlockRecord],
    last_timestamp: uint64,
    farmer_reward_puzzlehash: bytes32,
    pool_target: PoolTarget,
    prev_block: Optional[BlockRecord],
    block_generator: Optional[BlockGenerator],
    additions: list[Coin],
    removals: list[Coin],
) -> tuple[FullBlock, BlockRecord, float]:
    timestamp = last_timestamp + 20
    if prev_block is None:
        height: uint32 = uint32(0)
        prev_block_hash: bytes32 = constants.GENESIS_CHALLENGE
    else:
        height = uint32(prev_block.height + 1)
        prev_block_hash = prev_block.header_hash

    fees: uint64 = uint64(sum(c.amount for c in removals) - sum(c.amount for c in additions))

    if height > 0:
        assert prev_block is not None
        additions.append(
            create_pool_coin(
                prev_block.height,
                prev_block.pool_puzzle_hash,
                calculate_pool_reward(prev_block.height),
                constants.GENESIS_CHALLENGE,
            )
        )
        additions.append(
            create_farmer_coin(
                prev_block.height,
                prev_block.farmer_puzzle_hash,
                uint64(
                    calculate_base_farmer_reward(prev_block.height) + prev_block.fees
                    if prev_block.fees is not None
                    else 0
                ),
                constants.GENESIS_CHALLENGE,
            )
        )

    byte_array_tx: list[bytearray] = []
    removal_ids: list[bytes32] = []
    puzzlehash_coin_map: dict[bytes32, list[bytes32]] = {}
    for coin in additions:
        puzzlehash_coin_map.setdefault(coin.puzzle_hash, [])
        puzzlehash_coin_map[coin.puzzle_hash].append(coin.name())
        byte_array_tx.append(bytearray(coin.puzzle_hash))
    for coin in removals:
        cname = coin.name()
        removal_ids.append(cname)
        byte_array_tx.append(bytearray(cname))
    bip158: PyBIP158 = PyBIP158(byte_array_tx)
    filter_hash = std_hash(bytes(bip158.GetEncoded()))

    additions_merkle_items: list[bytes32] = []
    for puzzle, coin_ids in puzzlehash_coin_map.items():
        additions_merkle_items.append(puzzle)
        additions_merkle_items.append(hash_coin_ids(coin_ids))

    additions_root = bytes32(compute_merkle_set_root(additions_merkle_items))
    removals_root = bytes32(compute_merkle_set_root(removal_ids))

    generator_hash = bytes32.zeros
    if block_generator is not None:
        generator_hash = std_hash(block_generator.program)

    foliage_data = FoliageBlockData(
        bytes32.zeros,
        pool_target,
        G2Element(),
        farmer_reward_puzzlehash,
        bytes32.zeros,
    )

    transactions_info = TransactionsInfo(
        generator_hash,
        bytes32.zeros,
        G2Element(),
        fees,
        uint64(constants.MAX_BLOCK_COST_KLVM),
        additions[-2:],
    )

    foliage_transaction_block = FoliageTransactionBlock(
        prev_block_hash,
        uint64(timestamp),
        filter_hash,
        additions_root,
        removals_root,
        transactions_info.get_hash(),
    )

    foliage = Foliage(
        prev_block_hash,
        bytes32.zeros,
        foliage_data,
        G2Element(),
        foliage_transaction_block.get_hash(),
        G2Element(),
    )
    unfinished_block = UnfinishedBlock(
        [],
        RewardChainBlockUnfinished(
            uint128(1),
            uint8(1),
            bytes32.zeros,
            DEFAULT_PROOF_OF_SPACE,
            None,
            G2Element(),
            None,
            G2Element(),
        ),
        None,
        None,
        foliage,
        foliage_transaction_block,
        transactions_info,
        block_generator.program if block_generator else None,
        [],
    )

    full_block, block_record = finish_block(constants, unfinished_block, prev_block, blocks)

    return full_block, block_record, timestamp



from __future__ import annotations

import random
import time
from dataclasses import dataclass
from typing import Optional

from chik.types.peer_info import PeerInfo
from chik.util.ints import uint16, uint64


@dataclass(frozen=False)
class VettedPeer:
    host: str
    port: uint16

    # 0 means we have not attempted to vet this peer yet
    # a negative number means we have failed that many vetting attempts in a row
    # a positive number means we have successfully vetted the peer this many
    # times in a row
    vetted: int = 0
    # the timestamp of the last *successful* vetting of this peer
    vetted_timestamp: uint64 = uint64(0)
    # the last time we attempted to vet this peer, or 0 if we haven't tried yet
    # we set this regardless of whether the vetting is successful or not
    last_attempt: uint64 = uint64(0)
    time_added: uint64 = uint64(0)

    def __eq__(self, rhs: object) -> bool:
        return self.host == rhs.host and self.port == rhs.port  # type: ignore[no-any-return, attr-defined]

    def __hash__(self) -> int:
        return hash((self.host, self.port))


class IntroducerPeers:
    """
    Has the list of known full node peers that are already connected or may be
    connected to, and the time that they were last added.
    """

    def __init__(self) -> None:
        self._peers: set[VettedPeer] = set()

    def add(self, peer: Optional[PeerInfo]) -> bool:
        if peer is None or not peer.port:
            return False

        p = VettedPeer(peer.host, peer.port)
        p.time_added = uint64(int(time.time()))

        if p in self._peers:
            return True

        self._peers.add(p)
        return True

    def remove(self, peer: Optional[VettedPeer]) -> bool:
        if peer is None or not peer.port:
            return False
        try:
            self._peers.remove(peer)
            return True
        except ValueError:
            return False

    def get_peers(
        self, max_peers: int = 0, randomize: bool = False, recent_threshold: float = 9999999
    ) -> list[VettedPeer]:
        target_peers = [peer for peer in self._peers if time.time() - float(peer.time_added) < recent_threshold]
        if not max_peers or max_peers > len(target_peers):
            max_peers = len(target_peers)
        if randomize:
            return random.sample(target_peers, max_peers)
        else:
            return target_peers[:max_peers]



from __future__ import annotations

from dataclasses import dataclass
from typing import Union

from chik_rs import G1Element

from chik.types.blockchain_format.sized_bytes import bytes32
from chik.util.ints import uint32
from chik.wallet.util.wallet_types import WalletType


@dataclass(frozen=True)
class DerivationRecord:
    """
    These are records representing a puzzle hash, which is generated from a
    public key, derivation index, and wallet type. Stored in the puzzle_store.
    """

    index: uint32
    puzzle_hash: bytes32
    _pubkey: Union[G1Element, bytes]
    wallet_type: WalletType
    wallet_id: uint32
    hardened: bool

    @property
    def pubkey(self) -> G1Element:
        assert isinstance(self._pubkey, G1Element)
        return self._pubkey



#!/usr/bin/env python3

from __future__ import annotations

import asyncio
import os
from pathlib import Path

import pytest

from chik._tests.util.full_sync import run_sync_test


@pytest.mark.parametrize("keep_up", [True, False])
def test_full_sync_test(keep_up: bool) -> None:
    file_path = os.path.realpath(__file__)
    db_file = Path(file_path).parent / "test-blockchain-db.sqlite"
    asyncio.run(
        run_sync_test(
            db_file,
            db_version=2,
            profile=False,
            single_thread=False,
            test_constants=False,
            keep_up=keep_up,
            db_sync="off",
            node_profiler=False,
            start_at_checkpoint=None,
        )
    )
